@@ -22,6 +22,8 @@
 
 #include <comdef.h>
 #include <Lmcons.h>
+#include <psapi.h>
+#include <RestartManager.h>
 #include <shlguid.h>
 #include <shlobj.h>
 #include <string>
@@ -43,6 +45,72 @@ static const char runPathC[] = R"(HKEY_CURRENT_USER\Software\Microsoft\Windows\C
 
 namespace OCC {
 
+QVector<Utility::ProcessInfosForOpenFile> Utility::queryProcessInfosKeepingFileOpen(const QString &filePath)
+{
+    QVector<ProcessInfosForOpenFile> results;
+
+    DWORD restartManagerSession = 0;
+    WCHAR restartManagerSessionKey[CCH_RM_SESSION_KEY + 1] = {0};
+    auto errorStatus = RmStartSession(&restartManagerSession, 0, restartManagerSessionKey);
+    if (errorStatus != ERROR_SUCCESS) {
+        return results;
+    }
+
+    LPCWSTR files[] = {reinterpret_cast<LPCWSTR>(filePath.utf16())};
+    errorStatus = RmRegisterResources(restartManagerSession, 1, files, 0, NULL, 0, NULL);
+    if (errorStatus != ERROR_SUCCESS) {
+        RmEndSession(restartManagerSession);
+        return results;
+    }
+
+    DWORD rebootReasons = 0;
+    UINT rmProcessInfosNeededCount = 0;
+    std::vector<RM_PROCESS_INFO> rmProcessInfos;
+    auto rmProcessInfosRequestedCount = static_cast<UINT>(rmProcessInfos.size());
+    errorStatus = RmGetList(restartManagerSession, &rmProcessInfosNeededCount, &rmProcessInfosRequestedCount, rmProcessInfos.data(), &rebootReasons);
+    
+    if (errorStatus == ERROR_MORE_DATA) {
+        rmProcessInfos.resize(rmProcessInfosNeededCount, {});
+        rmProcessInfosRequestedCount = static_cast<UINT>(rmProcessInfos.size());
+        errorStatus = RmGetList(restartManagerSession, &rmProcessInfosNeededCount, &rmProcessInfosRequestedCount, rmProcessInfos.data(), &rebootReasons);
+    }
+    
+    if (errorStatus != ERROR_SUCCESS || rmProcessInfos.empty()) {
+        RmEndSession(restartManagerSession);
+        return results;
+    }
+
+    for (size_t i = 0; i < rmProcessInfos.size(); ++i) {
+        const auto processHandle = OpenProcess(PROCESS_QUERY_LIMITED_INFORMATION, FALSE, rmProcessInfos[i].Process.dwProcessId);
+        if (!processHandle) {
+            continue;
+        }
+
+        FILETIME ftCreate, ftExit, ftKernel, ftUser;
+
+        if (!GetProcessTimes(processHandle, &ftCreate, &ftExit, &ftKernel, &ftUser)
+            || CompareFileTime(&rmProcessInfos[i].Process.ProcessStartTime, &ftCreate) != 0) {
+            CloseHandle(processHandle);
+            continue;
+        }
+
+        WCHAR processFullPath[MAX_PATH];
+        DWORD processFullPathLength = MAX_PATH;
+        if (QueryFullProcessImageNameW(processHandle, 0, processFullPath, &processFullPathLength) && processFullPathLength <= MAX_PATH) {
+            const auto processFullPathString = QDir::fromNativeSeparators(QString::fromWCharArray(processFullPath));
+            const QFileInfo fileInfoForProcess(processFullPathString);
+            const auto processName = fileInfoForProcess.fileName();
+            if (!processName.isEmpty()) {
+                results.push_back(Utility::ProcessInfosForOpenFile{rmProcessInfos[i].Process.dwProcessId, processName});
+            }
+        }
+        CloseHandle(processHandle);
+    }
+    RmEndSession(restartManagerSession);
+
+    return results;
+}
+
 void Utility::setupFavLink(const QString &folder)
 {
     // First create a Desktop.ini so that the folder and favorite link show our application's icon.
