@@ -27,6 +27,10 @@
 #include "owncloudgui.h"
 #include "folderstatusmodel.h"
 
+#ifdef BUILD_FILE_PROVIDER_MODULE
+#include "macOS/fileprovidersettingscontroller.h"
+#endif
+
 class QModelIndex;
 class QNetworkReply;
 class QListWidgetItem;
@@ -118,7 +122,7 @@ private slots:
 
 private slots:
     void displayMnemonic(const QString &mnemonic);
-    void disableEncryptionForAccount(const AccountPtr &account) const;
+    void disableEncryptionForAccount(const OCC::AccountPtr &account) const;
     void showConnectionLabel(const QString &message, QStringList errors = QStringList());
     void openIgnoredFilesDialog(const QString & absFolderPath);
     void customizeStyle();
@@ -132,6 +136,8 @@ private:
     bool event(QEvent *) override;
     QAction *addActionToEncryptionMessage(const QString &actionTitle, const QString &actionId);
 
+    void initializeE2eEncryptionSettingsMessage();
+
     /// Returns the alias of the selected folder, empty string if none
     [[nodiscard]] QString selectedFolderAlias() const;
 
