@@ -13,6 +13,7 @@
  */
 
 #include "accountstate.h"
+
 #include "accountmanager.h"
 #include "remotewipe.h"
 #include "account.h"
@@ -23,6 +24,7 @@
 #include "ocsnavigationappsjob.h"
 #include "ocsuserstatusconnector.h"
 #include "pushnotifications.h"
+#include "networkjobs.h"
 
 #include <QSettings>
 #include <QTimer>
@@ -32,6 +34,7 @@
 #include <QJsonArray>
 #include <QNetworkRequest>
 #include <QBuffer>
+#include <QRandomGenerator>
 
 #include <cmath>
 
@@ -39,13 +42,14 @@ namespace OCC {
 
 Q_LOGGING_CATEGORY(lcAccountState, "nextcloud.gui.account.state", QtInfoMsg)
 
-AccountState::AccountState(AccountPtr account)
+AccountState::AccountState(const AccountPtr &account)
     : QObject()
     , _account(account)
     , _state(AccountState::Disconnected)
     , _connectionStatus(ConnectionValidator::Undefined)
     , _waitingForNewCredentials(false)
-    , _maintenanceToConnectedDelay(60000 + (qrand() % (4 * 60000))) // 1-5min delay
+    , _termsOfServiceChecker(_account)
+    , _maintenanceToConnectedDelay(60000 + (QRandomGenerator::global()->generate() % (4 * 60000))) // 1-5min delay
     , _remoteWipe(new RemoteWipe(_account))
     , _isDesktopNotificationsAllowed(true)
 {
@@ -61,6 +65,18 @@ AccountState::AccountState(AccountPtr account)
             this, &AccountState::slotPushNotificationsReady);
     connect(account.data(), &Account::serverUserStatusChanged, this,
         &AccountState::slotServerUserStatusChanged);
+    connect(&_termsOfServiceChecker, &TermsOfServiceChecker::done,
+            this, [this] ()
+            {
+                if (_termsOfServiceChecker.needToSign()) {
+                    slotConnectionValidatorResult(ConnectionValidator::NeedToSignTermsOfService, {});
+                }
+            });
+    connect(account.data(), &Account::termsOfServiceNeedToBeChecked,
+            this, [this] ()
+            {
+                _termsOfServiceChecker.start();
+            });
 
     connect(this, &AccountState::isConnectedChanged, [=]{
         // Get the Apps available on the server if we're now connected.
@@ -73,21 +89,15 @@ AccountState::AccountState(AccountPtr account)
     _checkConnectionTimer.setInterval(ConnectionValidator::DefaultCallingIntervalMsec);
     _checkConnectionTimer.start();
 
+    connect(&_checkServerAvailibilityTimer, &QTimer::timeout, this, &AccountState::slotCheckServerAvailibility);
+    _checkServerAvailibilityTimer.setInterval(ConnectionValidator::DefaultCallingIntervalMsec);
+    _checkServerAvailibilityTimer.start();
+
     QTimer::singleShot(0, this, &AccountState::slotCheckConnection);
 }
 
 AccountState::~AccountState() = default;
 
-AccountState *AccountState::loadFromSettings(AccountPtr account, QSettings & /*settings*/)
-{
-    auto accountState = new AccountState(account);
-    return accountState;
-}
-
-void AccountState::writeToSettings(QSettings & /*settings*/)
-{
-}
-
 AccountPtr AccountState::account() const
 {
     return _account;
@@ -118,15 +128,14 @@ void AccountState::setState(State state)
 
         if (_state == SignedOut) {
             _connectionStatus = ConnectionValidator::Undefined;
-            _connectionErrors.clear();
         } else if (oldState == SignedOut && _state == Disconnected) {
             // If we stop being voluntarily signed-out, try to connect and
             // auth right now!
             checkConnectivity();
-        } else if (_state == ServiceUnavailable) {
-            // Check if we are actually down for maintenance.
+        } else if (_state == ServiceUnavailable || _state == RedirectDetected) {
+            // Check if we are actually down for maintenance/in a redirect state (captive portal?).
             // To do this we must clear the connection validator that just
-            // produced the 503. It's finished anyway and will delete itself.
+            // produced the 503/302. It's finished anyway and will delete itself.
             _connectionValidator.clear();
             checkConnectivity();
         }
@@ -155,12 +164,16 @@ QString AccountState::stateString(State state)
         return tr("Service unavailable");
     case MaintenanceMode:
         return tr("Maintenance mode");
+    case RedirectDetected:
+        return tr("Redirect detected");
     case NetworkError:
         return tr("Network error");
     case ConfigurationError:
         return tr("Configuration error");
     case AskingCredentials:
         return tr("Asking Credentials");
+    case NeedToSignTermsOfService:
+        return tr("Need the user to accept the terms of service");
     }
     return tr("Unknown account state");
 }
@@ -296,8 +309,9 @@ void AccountState::checkConnectivity()
         return;
     }
 
-    auto *conValidator = new ConnectionValidator(AccountStatePtr(this));
+    auto *conValidator = new ConnectionValidator(AccountStatePtr(this), _connectionErrors);
     _connectionValidator = conValidator;
+    _connectionErrors.clear();
     connect(conValidator, &ConnectionValidator::connectionResult,
         this, &AccountState::slotConnectionValidatorResult);
     if (isConnected()) {
@@ -318,7 +332,7 @@ void AccountState::checkConnectivity()
 
         // If we don't reset the ssl config a second CheckServerJob can produce a
         // ssl config that does not have a sensible certificate chain.
-        account()->setSslConfiguration(QSslConfiguration());
+        account()->setSslConfiguration(QSslConfiguration::defaultConfiguration());
         //#endif
         conValidator->checkServerAndAuth();
     }
@@ -345,12 +359,14 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         return;
     }
 
+    const auto oldConnectionValidatorStatus = _lastConnectionValidatorStatus;
     _lastConnectionValidatorStatus = status;
 
-    // Come online gradually from 503 or maintenance mode
+    // Come online gradually from 503, captive portal(redirection) or maintenance mode
     if (status == ConnectionValidator::Connected
         && (_connectionStatus == ConnectionValidator::ServiceUnavailable
-            || _connectionStatus == ConnectionValidator::MaintenanceMode)) {
+            || _connectionStatus == ConnectionValidator::MaintenanceMode
+              || _connectionStatus == ConnectionValidator::StatusRedirect)) {
         if (!_timeSinceMaintenanceOver.isValid()) {
             qCInfo(lcAccountState) << "AccountState reconnection: delaying for"
                                    << _maintenanceToConnectedDelay << "ms";
@@ -369,6 +385,7 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
                                << _connectionStatus << "->"
                                << status;
         _connectionStatus = status;
+        emit stateChanged(_state);
     }
     _connectionErrors = errors;
 
@@ -415,10 +432,23 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         _timeSinceMaintenanceOver.invalidate();
         setState(MaintenanceMode);
         break;
+    case ConnectionValidator::StatusRedirect:
+        _timeSinceMaintenanceOver.invalidate();
+        setState(RedirectDetected);
+        break;
     case ConnectionValidator::Timeout:
         setState(NetworkError);
         updateRetryCount();
         break;
+    case ConnectionValidator::NeedToSignTermsOfService:
+        setState(NeedToSignTermsOfService);
+        break;
+    }
+
+    if ((oldConnectionValidatorStatus == ConnectionValidator::NeedToSignTermsOfService && status == ConnectionValidator::Connected) ||
+        (status == ConnectionValidator::NeedToSignTermsOfService && oldConnectionValidatorStatus != status)) {
+
+        emit termsOfServiceChanged(_account, status == ConnectionValidator::NeedToSignTermsOfService ? AccountState::NeedToSignTermsOfService : AccountState::Connected);
     }
 }
 
@@ -449,10 +479,6 @@ void AccountState::handleInvalidCredentials()
     if (account()->credentials()->ready()) {
         account()->credentials()->invalidateToken();
     }
-    if (auto creds = qobject_cast<HttpCredentials *>(account()->credentials())) {
-        if (creds->refreshAccessToken())
-            return;
-    }
     account()->credentials()->askFromUser();
 }
 
@@ -554,6 +580,28 @@ void AccountState::slotCheckConnection()
     }
 }
 
+void AccountState::slotCheckServerAvailibility()
+{
+    if (state() == AccountState::Connected
+        || state() == AccountState::SignedOut
+        || state() == AccountState::MaintenanceMode
+        || state() == AccountState::AskingCredentials) {
+        qCInfo(lcAccountState) << "Skipping server availability check for account" << _account->davUser() << "with state" << state();
+        return;
+    }
+    qCInfo(lcAccountState) << "Checking server availability for account" << _account->davUser();
+    const auto serverAvailibilityUrl = Utility::concatUrlPath(_account->url(), QLatin1String("/index.php/204"));
+    auto checkServerAvailibilityJob = _account->sendRequest(QByteArrayLiteral("GET"), serverAvailibilityUrl);
+    connect(checkServerAvailibilityJob, &SimpleNetworkJob::finishedSignal, this, [this](QNetworkReply *reply) {
+        if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 204) {
+            qCInfo(lcAccountState) << "Server is now available for account" << _account->davUser();
+            _lastCheckConnectionTimer.invalidate();
+            resetRetryCount();
+            QMetaObject::invokeMethod(this, &AccountState::slotCheckConnection, Qt::QueuedConnection);
+        }
+    });
+}
+
 void AccountState::slotPushNotificationsReady()
 {
     if (state() != AccountState::State::Connected) {
