@@ -17,10 +17,15 @@
 #include <QFileInfo>
 #include <QTimeZone>
 
+#include <array>
+#include <random>
+#include <openssl/rand.h>
+
 #include "account.h"
 #include "folderman.h"
+#include "sharepermissions.h"
 #include "theme.h"
-#include "wordlist.h"
+#include "updatee2eefolderusersmetadatajob.h"
 
 namespace {
 
@@ -28,16 +33,6 @@ static const auto placeholderLinkShareId = QStringLiteral("__placeholderLinkShar
 static const auto internalLinkShareId = QStringLiteral("__internalLinkShareId__");
 static const auto secureFileDropPlaceholderLinkShareId = QStringLiteral("__secureFileDropPlaceholderLinkShareId__");
 
-QString createRandomPassword()
-{
-    const auto words = OCC::WordList::getRandomWords(10);
-
-    const auto addFirstLetter = [](const QString &current, const QString &next) -> QString {
-        return current + next.at(0);
-    };
-
-    return std::accumulate(std::cbegin(words), std::cend(words), QString(), addFirstLetter);
-}
 }
 
 namespace OCC
@@ -86,6 +81,7 @@ QHash<int, QByteArray> ShareModel::roleNames() const
     roles[IsSharePermissionsChangeInProgress] = "isSharePermissionChangeInProgress";
     roles[HideDownloadEnabledRole] = "hideDownload";
     roles[IsHideDownloadEnabledChangeInProgress] = "isHideDownloadInProgress";
+    roles[ResharingAllowedRole] = "resharingAllowed";
 
     return roles;
 }
@@ -94,7 +90,8 @@ QVariant ShareModel::data(const QModelIndex &index, const int role) const
 {
     Q_ASSERT(checkIndex(index, QAbstractItemModel::CheckIndexOption::IndexIsValid | QAbstractItemModel::CheckIndexOption::ParentIsInvalid));
 
-    const auto share = _shares.at(index.row());
+    const auto shareIdx = index.row();
+    const auto share = _shares.at(shareIdx);
 
     if (!share) {
         return {};
@@ -142,7 +139,7 @@ QVariant ShareModel::data(const QModelIndex &index, const int role) const
 
     switch (role) {
     case Qt::DisplayRole:
-        return displayStringForShare(share);
+        return displayStringForShare(share, _duplicateDisplayNameShareIndices.contains(shareIdx));
     case ShareRole:
         return QVariant::fromValue(share);
     case ShareTypeRole:
@@ -187,9 +184,13 @@ QVariant ShareModel::data(const QModelIndex &index, const int role) const
     case EditingAllowedRole:
         return share->getPermissions().testFlag(SharePermissionUpdate);
 
+    case ResharingAllowedRole:
+        return share->getPermissions().testFlag(SharePermissionShare);
+
     // Deal with roles that only return certain values for link or user/group share types
     case NoteEnabledRole:
     case ExpireDateEnabledRole:
+    case HideDownloadEnabledRole:
         return false;
     case LinkRole:
     case LinkShareNameRole:
@@ -199,7 +200,6 @@ QVariant ShareModel::data(const QModelIndex &index, const int role) const
         return {};
     }
 
-    qCWarning(lcShareModel) << "Got unknown role" << role << "for share of type" << share->getShareType() << "so returning null value.";
     return {};
 }
 
@@ -209,7 +209,7 @@ void ShareModel::resetData()
 {
     beginResetModel();
 
-    _folder = nullptr;
+    _synchronizationFolder = nullptr;
     _sharePath.clear();
     _maxSharingPermissions = {};
     _numericFileId.clear();
@@ -243,9 +243,9 @@ void ShareModel::updateData()
         return;
     }
 
-    _folder = FolderMan::instance()->folderForPath(_localPath);
+    _synchronizationFolder = FolderMan::instance()->folderForPath(_localPath);
 
-    if (!_folder) {
+    if (!_synchronizationFolder) {
         qCWarning(lcShareModel) << "Could not update share model data for" << _localPath << "no responsible folder found";
         resetData();
         return;
@@ -253,18 +253,24 @@ void ShareModel::updateData()
 
     qCDebug(lcShareModel) << "Updating share model data now.";
 
-    const auto relPath = _localPath.mid(_folder->cleanPath().length() + 1);
-    _sharePath = _folder->remotePathTrailingSlash() + relPath;
+    const auto relPath = _localPath.mid(_synchronizationFolder->cleanPath().length() + 1);
+    _sharePath = _synchronizationFolder->remotePathTrailingSlash() + relPath;
 
     SyncJournalFileRecord fileRecord;
     auto resharingAllowed = true; // lets assume the good
 
-    if (_folder->journalDb()->getFileRecord(relPath, &fileRecord) && fileRecord.isValid() && !fileRecord._remotePerm.isNull()
+    if (_synchronizationFolder->journalDb()->getFileRecord(relPath, &fileRecord) && fileRecord.isValid() && !fileRecord._remotePerm.isNull()
         && !fileRecord._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
         qCInfo(lcShareModel) << "File record says resharing not allowed";
         resharingAllowed = false;
     }
 
+    if (fileRecord.isVirtualFile() && _synchronizationFolder->vfs().mode() == Vfs::WithSuffix) {
+        if (const auto suffix = _synchronizationFolder->vfs().fileSuffix(); !suffix.isEmpty() && _sharePath.endsWith(suffix)) {
+            _sharePath.chop(suffix.length());
+        }
+    }
+
     _maxSharingPermissions = resharingAllowed ? SharePermissions(_accountState->account()->capabilities().shareDefaultPermissions()) : SharePermissions({});
     Q_EMIT sharePermissionsChanged();
 
@@ -280,6 +286,14 @@ void ShareModel::updateData()
         _sharedItemType = fileRecord.isE2eEncrypted() ? SharedItemType::SharedItemTypeEncryptedFile : SharedItemType::SharedItemTypeFile;
     }
 
+    const auto prevIsShareDisabledEncryptedFolder = _isShareDisabledEncryptedFolder;
+    _isShareDisabledEncryptedFolder = fileRecord.isE2eEncrypted()
+        && (_sharedItemType != SharedItemType::SharedItemTypeEncryptedTopLevelFolder
+            || fileRecord._e2eEncryptionStatus < SyncJournalFileRecord::EncryptionStatus::EncryptedMigratedV2_0);
+    if (prevIsShareDisabledEncryptedFolder != _isShareDisabledEncryptedFolder) {
+        emit isShareDisabledEncryptedFolderChanged();
+    }
+
     // Will get added when shares are fetched if no link shares are fetched
     _placeholderLinkShare.reset(new Share(_accountState->account(),
                                           placeholderLinkShareId,
@@ -440,14 +454,14 @@ void ShareModel::slotPropfindReceived(const QVariantMap &result)
     }
 
     const auto privateLinkUrl = result["privatelink"].toString();
-    const auto numericFileId = result["fileid"].toByteArray();
+    _fileRemoteId = result["fileid"].toByteArray();
 
     if (!privateLinkUrl.isEmpty()) {
         qCInfo(lcShareModel) << "Received private link url for" << _sharePath << privateLinkUrl;
         _privateLinkUrl = privateLinkUrl;
-    } else if (!numericFileId.isEmpty()) {
-        qCInfo(lcShareModel) << "Received numeric file id for" << _sharePath << numericFileId;
-        _privateLinkUrl = _accountState->account()->deprecatedPrivateLinkUrl(numericFileId).toString(QUrl::FullyEncoded);
+    } else if (!_fileRemoteId.isEmpty()) {
+        qCInfo(lcShareModel) << "Received numeric file id for" << _sharePath << _fileRemoteId;
+        _privateLinkUrl = _accountState->account()->deprecatedPrivateLinkUrl(_fileRemoteId).toString(QUrl::FullyEncoded);
     }
 
     setupInternalLinkShare();
@@ -473,6 +487,41 @@ void ShareModel::slotSharesFetched(const QList<SharePtr> &shares)
         slotAddShare(share);
     }
 
+    // Perform forward pass on shares and check for duplicate display names; store these indeces so
+    // we can check for these and display the specific user identifier in the display string later
+    _duplicateDisplayNameShareIndices.clear();
+    const auto shareCount = _shares.count();
+    for (auto i = 0; i < shareCount; ++i) {
+        if (_duplicateDisplayNameShareIndices.contains(i)) {
+            continue;
+        }
+
+        const auto sharee = _shares.at(i)->getShareWith();
+        if (sharee == nullptr) {
+            continue;
+        }
+
+        const auto duplicateIndices = QSharedPointer<QSet<unsigned int>>::create();
+        const auto handleDuplicateIndex = [this, duplicateIndices](const unsigned int idx) {
+            duplicateIndices->insert(idx);
+            _duplicateDisplayNameShareIndices[idx] = duplicateIndices;
+            const auto targetIdx = index(idx);
+            dataChanged(targetIdx, targetIdx, {Qt::DisplayRole});
+        };
+
+        for (auto j = i + 1; j < shareCount; ++j) {
+            const auto otherSharee = _shares.at(j)->getShareWith();
+            if (otherSharee == nullptr || sharee->format() != otherSharee->format()) {
+                continue;
+            }
+            handleDuplicateIndex(j);
+        }
+
+        if (!duplicateIndices->isEmpty()) {
+            handleDuplicateIndex(i);
+        }
+    }
+
     handleLinkShare();
 }
 
@@ -598,10 +647,26 @@ void ShareModel::slotRemoveShareWithId(const QString &shareId)
     const auto sharee = share->getShareWith();
     slotRemoveSharee(sharee);
 
-    beginRemoveRows({}, shareIndex.row(), shareIndex.row());
-    _shares.removeAt(shareIndex.row());
+    const auto shareRow = shareIndex.row();
+    beginRemoveRows({}, shareRow, shareRow);
+    _shares.removeAt(shareRow);
     endRemoveRows();
 
+    // Handle display name duplicates now. First remove the index from the bucket it was in; then,
+    // check if this removal means the remaining index in the bucket is no longer a duplicate.
+    // If this is the case then handle the update for this item too.
+    const auto duplicateShares = _duplicateDisplayNameShareIndices.value(shareRow);
+    if (duplicateShares) {
+        duplicateShares->remove(shareRow);
+        if (duplicateShares->count() == 1) {
+            const auto noLongerDuplicateIndex = *(duplicateShares->begin());
+            _duplicateDisplayNameShareIndices.remove(noLongerDuplicateIndex);
+            const auto noLongerDuplicateModelIndex = index(noLongerDuplicateIndex);
+            Q_EMIT dataChanged(noLongerDuplicateModelIndex, noLongerDuplicateModelIndex, {Qt::DisplayRole});
+        }
+        _duplicateDisplayNameShareIndices.remove(shareRow);
+    }
+
     handleLinkShare();
 
     Q_EMIT sharesChanged();
@@ -629,13 +694,13 @@ void ShareModel::slotRemoveSharee(const ShareePtr &sharee)
     Q_EMIT shareesChanged();
 }
 
-QString ShareModel::displayStringForShare(const SharePtr &share) const
+QString ShareModel::displayStringForShare(const SharePtr &share, const bool verbose) const
 {
     if (const auto linkShare = share.objectCast<LinkShare>()) {
 
         const auto isSecureFileDropShare = isSecureFileDropSupportedFolder() && linkShare->getPermissions().testFlag(OCC::SharePermission::SharePermissionCreate);
 
-        const auto displayString = isSecureFileDropShare ? tr("Secure filedrop link") : tr("Share link");
+        const auto displayString = isSecureFileDropShare ? tr("Secure file drop link") : tr("Share link");
 
         if (!linkShare->getLabel().isEmpty()) {
             return QStringLiteral("%1 (%2)").arg(displayString, linkShare->getLabel());
@@ -649,7 +714,8 @@ QString ShareModel::displayStringForShare(const SharePtr &share) const
     } else if (share->getShareType() == Share::TypeSecureFileDropPlaceholderLink) {
         return tr("Secure file drop");
     } else if (share->getShareWith()) {
-        return share->getShareWith()->format();
+        const auto sharee = share->getShareWith();
+        return verbose ? QString{"%1 (%2)"}.arg(sharee->format(), sharee->shareWith()) : sharee->format();
     }
 
     qCWarning(lcShareModel) << "Unable to provide good display string for share";
@@ -830,6 +896,47 @@ void ShareModel::slotShareExpireDateSet(const QString &shareId)
     Q_EMIT dataChanged(shareModelIndex, shareModelIndex, { ExpireDateEnabledRole, ExpireDateRole });
 }
 
+void ShareModel::slotDeleteE2EeShare(const SharePtr &share) const
+{
+    const auto account = accountState()->account();
+    QString folderAlias;
+    for (const auto &f : FolderMan::instance()->map()) {
+        if (f->accountState()->account() != account) {
+            continue;
+        }
+        const auto folderPath = f->remotePath();
+        if (share->path().startsWith(folderPath) && (share->path() == folderPath || folderPath.endsWith('/') || share->path()[folderPath.size()] == '/')) {
+            folderAlias = f->alias();
+        }
+    }
+
+    auto folder = FolderMan::instance()->folder(folderAlias);
+    if (!folder || !folder->journalDb()) {
+        emit serverError(404, tr("Could not find local folder for %1").arg(share->path()));
+        return;
+    }
+
+    Q_ASSERT(folder->remotePath() == QStringLiteral("/")
+             || Utility::noLeadingSlashPath(share->path()).startsWith(Utility::noLeadingSlashPath(Utility::noTrailingSlashPath(folder->remotePath()))));
+
+    const auto removeE2eeShareJob = new UpdateE2eeFolderUsersMetadataJob(account,
+                                                                         folder->journalDb(),
+                                                                         folder->remotePath(),
+                                                                         UpdateE2eeFolderUsersMetadataJob::Remove,
+                                                                         share->path(),
+                                                                         share->getShareWith()->shareWith());
+    removeE2eeShareJob->setParent(_manager.data());
+    removeE2eeShareJob->start();
+    connect(removeE2eeShareJob, &UpdateE2eeFolderUsersMetadataJob::finished, this, [share, this](int code, const QString &message) {
+        if (code != 200) {
+            qCWarning(lcShareModel) << "Could not remove share from E2EE folder's metadata!";
+            emit serverError(code, message);
+            return;
+        }
+        share->deleteShare();
+    });
+}
+
 // ----------------------- Shares modification slots ----------------------- //
 
 void ShareModel::toggleShareAllowEditing(const SharePtr &share, const bool enable)
@@ -898,7 +1005,7 @@ void ShareModel::toggleSharePasswordProtect(const SharePtr &share, const bool en
         return;
     }
 
-    const auto randomPassword = createRandomPassword();
+    const auto randomPassword = generatePassword();
     _shareIdRecentlySetPasswords.insert(share->getId(), randomPassword);
     share->setPassword(randomPassword);
 }
@@ -950,7 +1057,7 @@ void ShareModel::toggleShareNoteToRecipientFromQml(const QVariant &share, const
     toggleShareNoteToRecipient(ptr, enable);
 }
 
-void ShareModel::changePermissionModeFromQml(const QVariant &share, const SharePermissionsMode permissionMode)
+void ShareModel::changePermissionModeFromQml(const QVariant &share, const OCC::ShareModel::SharePermissionsMode permissionMode)
 {
     const auto sharePtr = share.value<SharePtr>();
     if (sharePtr.isNull() || _sharePermissionsChangeInProgress) {
@@ -1070,7 +1177,7 @@ void ShareModel::createNewLinkShare() const
 
     if (_manager) {
         const auto askOptionalPassword = _accountState->account()->capabilities().sharePublicLinkAskOptionalPassword();
-        const auto password = askOptionalPassword ? createRandomPassword() : QString();
+        const auto password = askOptionalPassword ? generatePassword() : QString();
         if (isSecureFileDropSupportedFolder()) {
             _manager->createSecureFileDropShare(_sharePath, {}, password);
             return;
@@ -1104,11 +1211,15 @@ void ShareModel::createNewUserGroupShare(const ShareePtr &sharee)
         return;
     }
 
-    _manager->createShare(_sharePath,
-                          Share::ShareType(sharee->type()),
-                          sharee->shareWith(),
-                          _maxSharingPermissions,
-                          {});
+    if (isSecureFileDropSupportedFolder()) {
+        if (!_synchronizationFolder) {
+            qCWarning(lcShareModel) << "Could not share an E2EE folder" << _localPath << "no responsible folder found";
+            return;
+        }
+        _manager->createE2EeShareJob(_sharePath, sharee, _maxSharingPermissions, {});
+    } else {
+        _manager->createShare(_sharePath, Share::ShareType(sharee->type()), sharee->shareWith(), _maxSharingPermissions, {});
+    }
 }
 
 void ShareModel::createNewUserGroupShareWithPassword(const ShareePtr &sharee, const QString &password) const
@@ -1142,7 +1253,11 @@ void ShareModel::deleteShare(const SharePtr &share) const
         return;
     }
 
-    share->deleteShare();
+    if (isEncryptedItem() && Share::isShareTypeUserGroupEmailRoomOrRemote(share->getShareType())) {
+        slotDeleteE2EeShare(share);
+    } else {
+        share->deleteShare();
+    }
 }
 
 void ShareModel::deleteShareFromQml(const QVariant &share) const
@@ -1193,6 +1308,7 @@ void ShareModel::setAccountState(AccountState *accountState)
     Q_EMIT sharingEnabledChanged();
     Q_EMIT publicLinkSharesEnabledChanged();
     Q_EMIT userGroupSharingEnabledChanged();
+    Q_EMIT serverAllowsResharingChanged();
     updateData();
 }
 
@@ -1252,6 +1368,17 @@ bool ShareModel::canShare() const
     return _maxSharingPermissions & SharePermissionShare;
 }
 
+bool ShareModel::serverAllowsResharing() const
+{
+    return _accountState && _accountState->account() && _accountState->account()->capabilities().isValid()
+        && _accountState->account()->capabilities().shareResharing();
+}
+
+bool ShareModel::isShareDisabledEncryptedFolder() const
+{
+    return _isShareDisabledEncryptedFolder;
+}
+
 QVariantList ShareModel::sharees() const
 {
     QVariantList returnSharees;
@@ -1261,4 +1388,65 @@ QVariantList ShareModel::sharees() const
     return returnSharees;
 }
 
+QString ShareModel::generatePassword()
+{
+    constexpr auto asciiMin = 33;
+    constexpr auto asciiMax = 126;
+    constexpr auto asciiRange = asciiMax - asciiMin;
+    static constexpr auto numChars = 24;
+
+    static constexpr std::string_view lowercaseAlphabet = "abcdefghijklmnopqrstuvwxyz";
+    static constexpr std::string_view uppercaseAlphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    static constexpr std::string_view numbers = "0123456789";
+    static constexpr std::string_view specialChars = R"(ªº\\/|"'*+-_´¨{}·#$%&()=\[\]<>;:@~)";
+
+    static const QRegularExpression lowercaseMatch("[a-z]");
+    static const QRegularExpression uppercaseMatch("[A-Z]");
+    static const QRegularExpression numberMatch("[0-9]");
+    static const QRegularExpression specialCharMatch(QString("[%1]").arg(specialChars.data()));
+
+    static const std::map<std::string_view, QRegularExpression> matchMap{
+        {lowercaseAlphabet, lowercaseMatch},
+        {uppercaseAlphabet, uppercaseMatch},
+        {numbers, numberMatch},
+        {specialChars, specialCharMatch},
+    };
+
+    std::random_device rand_dev;
+    std::mt19937 rng(rand_dev());
+
+    QString passwd;
+    std::array<unsigned char, numChars> unsignedCharArray;
+
+    RAND_bytes(unsignedCharArray.data(), numChars);
+
+    for (const auto newChar : unsignedCharArray) {
+        // Ensure byte is within asciiRange
+        const auto byte = QChar((newChar % (asciiRange + 1)) + asciiMin);
+        passwd.append(byte);
+    }
+
+    for (const auto &charsWithMatcher : matchMap) {
+        const auto selectionChars = charsWithMatcher.first;
+        const auto matcher = charsWithMatcher.second;
+        Q_ASSERT(matcher.isValid());
+
+        if (matcher.match(passwd).hasMatch()) {
+            continue;
+        }
+
+        // add random required character at random position
+        std::uniform_int_distribution<std::mt19937::result_type> passwdDist(0, passwd.length() - 1);
+        std::uniform_int_distribution<std::mt19937::result_type> charsDist(0, selectionChars.length() - 1);
+
+        const auto passwdInsertIndex = passwdDist(rng);
+        const auto charToInsertIndex = charsDist(rng);
+        const auto charToInsert = selectionChars.at(charToInsertIndex);
+
+        passwd.insert(passwdInsertIndex, charToInsert);
+    }
+
+    return passwd;
+}
+
 } // namespace OCC
