@@ -27,6 +27,7 @@
 #include "gui/systray.h"
 #include <pushnotifications.h>
 #include <syncengine.h>
+#include "updatee2eefolderusersmetadatajob.h"
 
 #ifdef Q_OS_MAC
 #include <CoreServices/CoreServices.h>
@@ -172,9 +173,6 @@ int FolderMan::setupFolders()
 
     unloadAndDeleteAllFolders();
 
-    QStringList skipSettingsKeys;
-    backwardMigrationSettingsKeys(&skipSettingsKeys, &skipSettingsKeys);
-
     auto settings = ConfigFile::settingsWithGroup(QLatin1String("Accounts"));
     const auto accountsWithSettings = settings->childGroups();
     if (accountsWithSettings.isEmpty()) {
@@ -187,6 +185,9 @@ int FolderMan::setupFolders()
 
     qCInfo(lcFolderMan) << "Setup folders from settings file";
 
+    // this is done in Application::configVersionMigration
+    QStringList skipSettingsKeys;
+    backwardMigrationSettingsKeys(&skipSettingsKeys, &skipSettingsKeys);
     const auto accounts = AccountManager::instance()->accounts();
     for (const auto &account : accounts) {
         const auto id = account->account()->id();
@@ -353,18 +354,18 @@ int FolderMan::setupFoldersMigration()
     qCInfo(lcFolderMan) << "Setup folders from " << configPath << "(migration)";
 
     QDir dir(configPath);
-    //We need to include hidden files just in case the alias starts with '.'
+    // We need to include hidden files just in case the alias starts with '.'
     dir.setFilter(QDir::Files | QDir::Hidden);
-    const auto dirFiles = dir.entryList();
+    // Exclude previous backed up configs e.g. oc.cfg.backup_20230831_133749_4.0.0
+    // only need the current config in use by the legacy application
+    const auto dirFiles = dir.entryList({"*.cfg"});
 
-    // Normally there should be only one account when migrating. TODO: Change
-    const auto accountState = AccountManager::instance()->accounts().value(0).data();
+    // Migrate all folders for each account found in legacy config file(s)
+    const auto legacyAccounts = AccountManager::instance()->accounts();
     for (const auto &fileName : dirFiles) {
-        const auto fullFilePath = dir.filePath(fileName);
-        const auto folder = setupFolderFromOldConfigFile(fullFilePath, accountState);
-        if (folder) {
-            scheduleFolder(folder);
-            emit folderSyncStateChange(folder);
+        for (const auto &accountState : legacyAccounts) {
+            const auto fullFilePath = dir.filePath(fileName);
+            setupLegacyFolder(fullFilePath, accountState.data());
         }
     }
 
@@ -481,8 +482,7 @@ QString FolderMan::unescapeAlias(const QString &alias)
     return a;
 }
 
-// WARNING: Do not remove this code, it is used for predefined/automated deployments (2016)
-Folder *FolderMan::setupFolderFromOldConfigFile(const QString &fileNamePath, AccountState *accountState)
+void FolderMan::setupLegacyFolder(const QString &fileNamePath, AccountState *accountState)
 {
     qCInfo(lcFolderMan) << "  ` -> setting up:" << fileNamePath;
     QString escapedFileNamePath(fileNamePath);
@@ -498,7 +498,7 @@ Folder *FolderMan::setupFolderFromOldConfigFile(const QString &fileNamePath, Acc
     }
     if (!cfgFile.isReadable()) {
         qCWarning(lcFolderMan) << "Cannot read folder definition for alias " << cfgFile.filePath();
-        return nullptr;
+        return;
     }
 
     QSettings settings(escapedFileNamePath, QSettings::IniFormat);
@@ -509,88 +509,98 @@ Folder *FolderMan::setupFolderFromOldConfigFile(const QString &fileNamePath, Acc
     const auto groups = settings.childGroups();
     if (groups.isEmpty()) {
         qCWarning(lcFolderMan) << "empty file:" << cfgFile.filePath();
-        return nullptr;
+        return;
     }
 
     if (!accountState) {
         qCCritical(lcFolderMan) << "can't create folder without an account";
-        return nullptr;
+        return;
     }
 
     settings.beginGroup(settingsAccountsC);
-    const auto rootChildGroups = settings.childGroups();
-    for (const auto &accountId : rootChildGroups) {
-        qCDebug(lcFolderMan) << "try to migrate accountId:" << accountId;
-        settings.beginGroup(accountId);
-        settings.beginGroup(settingsFoldersC);
+    qCDebug(lcFolderMan) << "try to migrate accountId:" << accountState->account()->id();
+    settings.beginGroup(accountState->account()->id());
+    settings.beginGroup(settingsFoldersC);
 
-        if (settings.childGroups().isEmpty()) {
+    if (settings.childGroups().isEmpty()) {
+        qCDebug(lcFolderMan) << "there are no legacy folders for accountId:" << accountState->account()->id();
+        return;
+    }
+
+    const auto childGroups = settings.childGroups();
+    for (const auto &alias : childGroups) {
+        settings.beginGroup(alias);
+        qCDebug(lcFolderMan) << "try to migrate folder alias:" << alias;
+
+        const auto path = settings.value(QLatin1String("localPath")).toString();
+        const auto targetPath = settings.value(QLatin1String("targetPath")).toString();
+        const auto journalPath = settings.value(QLatin1String("journalPath")).toString();
+        const auto paused = settings.value(QLatin1String("paused"), false).toBool();
+        const auto ignoreHiddenFiles = settings.value(QLatin1String("ignoreHiddenFiles"), false).toBool();
+
+        if (path.isEmpty()) {
+            qCDebug(lcFolderMan) << "localPath is empty";
+            settings.endGroup();
             continue;
         }
 
-        const auto childGroups = settings.childGroups();
-        for (const auto &alias : childGroups) {
-            settings.beginGroup(alias);
-            qCDebug(lcFolderMan) << "try to migrate folder alias:" << alias;
-
-            const auto path = settings.value(QLatin1String("localPath")).toString();
-            const auto targetPath = settings.value(QLatin1String("targetPath")).toString();
-            const auto journalPath = settings.value(QLatin1String("journalPath")).toString();
-            const auto paused = settings.value(QLatin1String("paused"), false).toBool();
-            const auto ignoreHiddenFiles = settings.value(QLatin1String("ignoreHiddenFiles"), false).toBool();
-
-            if (path.isEmpty()) {
-                qCDebug(lcFolderMan) << "localPath is empty";
-                settings.endGroup();
-                continue;
-            }
-
-            if (targetPath.isEmpty()) {
-                qCDebug(lcFolderMan) << "targetPath is empty";
-                settings.endGroup();
-                continue;
-            }
-
-            if (journalPath.isEmpty()) {
-                qCDebug(lcFolderMan) << "journalPath is empty";
-                settings.endGroup();
-                continue;
-            }
-
-            FolderDefinition folderDefinition;
-            folderDefinition.alias = alias;
-            folderDefinition.localPath = path;
-            folderDefinition.targetPath = targetPath;
-            folderDefinition.journalPath = journalPath;
-            folderDefinition.paused = paused;
-            folderDefinition.ignoreHiddenFiles = ignoreHiddenFiles;
-
-            if (const auto folder = addFolderInternal(folderDefinition, accountState, std::make_unique<VfsOff>())) {
-                const auto blackList = settings.value(QLatin1String("blackList")).toStringList();
-                if (!blackList.empty()) {
-                    //migrate settings
-                    folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blackList);
-                    settings.remove(QLatin1String("blackList"));
-                    // FIXME: If you remove this codepath, you need to provide another way to do
-                    // this via theme.h or the normal FolderMan::setupFolders
-                }
-
-                folder->saveToSettings();
-
-                qCInfo(lcFolderMan) << "Migrated!" << folder;
-                settings.sync();
-
-                return folder;
-            }
-
+        if (targetPath.isEmpty()) {
+            qCDebug(lcFolderMan) << "targetPath is empty";
             settings.endGroup();
+            continue;
+        }
+
+        if (journalPath.isEmpty()) {
+            qCDebug(lcFolderMan) << "journalPath is empty";
+            settings.endGroup();
+            continue;
+        }
+
+        FolderDefinition folderDefinition;
+        folderDefinition.alias = alias;
+        folderDefinition.localPath = path;
+        folderDefinition.targetPath = targetPath;
+        folderDefinition.journalPath = journalPath;
+        folderDefinition.paused = paused;
+        folderDefinition.ignoreHiddenFiles = ignoreHiddenFiles;
+
+        if (const auto folder = addFolderInternal(folderDefinition, accountState, std::make_unique<VfsOff>())) {
+            auto ok = true;
+            auto legacyBlacklist = folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList,
+                                                                                   &ok);
+            if (!ok) {
+                qCInfo(lcFolderMan) << "There was a problem retrieving the database selective sync for " << folder;
+            }
+
+            legacyBlacklist << settings.value(QLatin1String("blackList")).toStringList();
+            if (!legacyBlacklist.isEmpty()) {
+                qCInfo(lcFolderMan) << "Legacy selective sync list found:" << legacyBlacklist;
+                for (const auto &legacyFolder : legacyBlacklist) {
+                    folder->migrateBlackListPath(legacyFolder);
+                }
+                settings.remove(QLatin1String("blackList"));
+            }
+
+            folder->saveToSettings();
+
+            qCInfo(lcFolderMan) << "Migrated!" << folder->path();
+            settings.sync();
+
+            if (!folder) {
+                continue;
+            }
+
+            scheduleFolder(folder);
+            emit folderSyncStateChange(folder);
         }
 
         settings.endGroup();
-        settings.endGroup();
     }
 
-    return nullptr;
+    settings.endGroup();
+    settings.endGroup();
+
+    return;
 }
 
 void FolderMan::slotFolderSyncPaused(Folder *f, bool paused)
@@ -754,7 +764,7 @@ void FolderMan::slotRunOneEtagJob()
         Folder *folder = nullptr;
         for (Folder *f : qAsConst(_folderMap)) {
             if (f->etagJob()) {
-                // Caveat: always grabs the first folder with a job, but we think this is Ok for now and avoids us having a seperate queue.
+                // Caveat: always grabs the first folder with a job, but we think this is Ok for now and avoids us having a separate queue.
                 _currentEtagJob = f->etagJob();
                 folder = f;
                 break;
@@ -987,7 +997,7 @@ void FolderMan::runEtagJobIfPossible(Folder *folder)
         return;
     }
     if (_scheduledFolders.contains(folder)) {
-        qCInfo(lcFolderMan) << "Can not run etag job: Folder is alreday scheduled";
+        qCInfo(lcFolderMan) << "Can not run etag job: Folder is already scheduled";
         return;
     }
     if (_disabledFolders.contains(folder)) {
@@ -1214,7 +1224,7 @@ Folder *FolderMan::addFolderInternal(
         || _folderMap.contains(folderDefinition.alias)
         || _additionalBlockedFolderAliases.contains(folderDefinition.alias)) {
         // There is already a folder configured with this name and folder names need to be unique
-        folderDefinition.alias = alias + QString::number(++count);
+        folderDefinition.alias = QString::number(alias.toInt() + (++count));
     }
 
     auto folder = new Folder(folderDefinition, accountState, std::move(vfs), this);
@@ -1259,6 +1269,38 @@ Folder *FolderMan::folderForPath(const QString &path)
     return it != folders.cend() ? *it : nullptr;
 }
 
+void FolderMan::addFolderToSelectiveSyncList(const QString &path, const SyncJournalDb::SelectiveSyncListType list)
+{
+    const auto folder = folderForPath(path);
+    if (!folder) {
+        return;
+    }
+
+    const QString folderPath = folder->cleanPath() + QLatin1Char('/');
+    const auto relPath = path.mid(folderPath.length());
+
+    switch (list) {
+    case SyncJournalDb::SelectiveSyncListType::SelectiveSyncWhiteList:
+        folder->whitelistPath(relPath);
+        break;
+    case SyncJournalDb::SelectiveSyncListType::SelectiveSyncBlackList:
+        folder->blacklistPath(relPath);
+        break;
+    default:
+        Q_UNREACHABLE();
+    }
+}
+
+void FolderMan::whitelistFolderPath(const QString &path)
+{
+    addFolderToSelectiveSyncList(path, SyncJournalDb::SelectiveSyncListType::SelectiveSyncWhiteList);
+}
+
+void FolderMan::blacklistFolderPath(const QString &path)
+{
+    addFolderToSelectiveSyncList(path, SyncJournalDb::SelectiveSyncListType::SelectiveSyncBlackList);
+}
+
 QStringList FolderMan::findFileInLocalFolders(const QString &relPath, const AccountPtr acc)
 {
     QStringList re;
@@ -1273,12 +1315,12 @@ QStringList FolderMan::findFileInLocalFolders(const QString &relPath, const Acco
         if (acc && folder->accountState()->account() != acc) {
             continue;
         }
-        if (!serverPath.startsWith(folder->remotePath()))
+        if (!serverPath.startsWith(folder->remotePathTrailingSlash()))
             continue;
 
         QString path = folder->cleanPath() + '/';
-        path += serverPath.midRef(folder->remotePathTrailingSlash().length());
-        if (QFile::exists(path)) {
+        path += serverPath.mid(folder->remotePathTrailingSlash().length());
+        if (FileSystem::fileExists(path)) {
             re.append(path);
         }
     }
@@ -1469,13 +1511,14 @@ void FolderMan::setDirtyProxy()
 {
     const auto folderMapValues = _folderMap.values();
     for (const auto folder : folderMapValues) {
-        if (folder) {
-            if (folder->accountState() && folder->accountState()->account()
-                && folder->accountState()->account()->networkAccessManager()) {
-                // Need to do this so we do not use the old determined system proxy
-                folder->accountState()->account()->networkAccessManager()->setProxy(
-                    QNetworkProxy(QNetworkProxy::DefaultProxy));
-            }
+        if (folder 
+            && folder->accountState() 
+            && folder->accountState()->account()
+            && folder->accountState()->account()->networkAccessManager()
+            && folder->accountState()->account()->networkProxySetting() == Account::AccountNetworkProxySetting::GlobalProxy) {
+            // Need to do this so we do not use the old determined system proxy
+            const auto proxy = QNetworkProxy(QNetworkProxy::DefaultProxy);
+            folder->accountState()->account()->networkAccessManager()->setProxy(proxy);
         }
     }
 }
@@ -1491,37 +1534,100 @@ void FolderMan::setDirtyNetworkLimits()
     }
 }
 
-void FolderMan::leaveShare(const QString &localFile)
+void FolderMan::setDirtyNetworkLimits(const AccountPtr &account) const
 {
-    if (const auto folder = FolderMan::instance()->folderForPath(localFile)) {
-        const auto filePathRelative = QString(localFile).remove(folder->path());
-
-        const auto leaveShareJob = new SimpleApiJob(folder->accountState()->account(), folder->accountState()->account()->davPath() + filePathRelative);
-        leaveShareJob->setVerb(SimpleApiJob::Verb::Delete);
-        connect(leaveShareJob, &SimpleApiJob::resultReceived, this, [this, folder](int statusCode) {
-            Q_UNUSED(statusCode)
-            scheduleFolder(folder);
-        });
-        leaveShareJob->start();
+    const auto folderMapValues = _folderMap.values();
+    for (const auto folder : folderMapValues) {
+        // set only in busy folders. Otherwise they read the config anyway.
+        if (folder && folder->isBusy() && folder->accountState()->account() == account) {
+            folder->setDirtyNetworkLimits();
+        }
     }
 }
 
+void FolderMan::leaveShare(const QString &localFile)
+{
+    const auto localFileNoTrailingSlash = localFile.endsWith('/') ? localFile.chopped(1) : localFile;
+    if (const auto folder = FolderMan::instance()->folderForPath(localFileNoTrailingSlash)) {
+        const auto filePathRelative = Utility::noLeadingSlashPath(QString(localFileNoTrailingSlash).remove(folder->path()));
+
+        SyncJournalFileRecord rec;
+        if (folder->journalDb()->getFileRecord(filePathRelative, &rec)
+            && rec.isValid() && rec.isE2eEncrypted()) {
+
+            if (_removeE2eeShareJob) {
+                _removeE2eeShareJob->deleteLater();
+            }
+
+            _removeE2eeShareJob = new UpdateE2eeFolderUsersMetadataJob(folder->accountState()->account(),
+                                                                                 folder->journalDb(),
+                                                                                 folder->remotePath(),
+                                                                                 UpdateE2eeFolderUsersMetadataJob::Remove,
+                                                                                 folder->remotePathTrailingSlash() + filePathRelative,
+                                                                                 folder->accountState()->account()->davUser());
+            _removeE2eeShareJob->setParent(this);
+            _removeE2eeShareJob->start(true);
+            connect(_removeE2eeShareJob, &UpdateE2eeFolderUsersMetadataJob::finished, this, [localFileNoTrailingSlash, this](int code, const QString &message) {   
+                if (code != 200) {
+                    qCDebug(lcFolderMan) << "Could not remove share from E2EE folder's metadata!" << code << message;
+                    return;
+                }
+                slotLeaveShare(localFileNoTrailingSlash, _removeE2eeShareJob->folderToken());
+            });
+
+            return;
+        }
+        slotLeaveShare(localFileNoTrailingSlash);
+    }
+}
+
+void FolderMan::slotLeaveShare(const QString &localFile, const QByteArray &folderToken)
+{
+    const auto folder = FolderMan::instance()->folderForPath(localFile);
+
+    if (!folder) {
+        qCWarning(lcFolderMan) << "Could not find a folder for localFile:" << localFile;
+        return;
+    }
+
+    const auto filePathRelative = QString(localFile).remove(folder->path());
+    const auto leaveShareJob = new SimpleApiJob(folder->accountState()->account(), folder->accountState()->account()->davPath() + filePathRelative);
+    leaveShareJob->setVerb(SimpleApiJob::Verb::Delete);
+    leaveShareJob->addRawHeader("e2e-token", folderToken);
+    connect(leaveShareJob, &SimpleApiJob::resultReceived, this, [this, folder, localFile](int statusCode) {
+        qCDebug(lcFolderMan) << "slotLeaveShare callback statusCode" << statusCode;
+        Q_UNUSED(statusCode);
+        if (_removeE2eeShareJob) {
+            _removeE2eeShareJob->unlockFolder(EncryptedFolderMetadataHandler::UnlockFolderWithResult::Success);
+            connect(_removeE2eeShareJob.data(), &UpdateE2eeFolderUsersMetadataJob::folderUnlocked, this, [this, folder] {
+                scheduleFolder(folder);
+            });
+            return;
+        }
+        scheduleFolder(folder);
+    });
+    leaveShareJob->start();
+}
+
 void FolderMan::trayOverallStatus(const QList<Folder *> &folders,
-    SyncResult::Status *status, bool *unresolvedConflicts)
+                                  SyncResult::Status *status,
+                                  bool *unresolvedConflicts,
+                                  ProgressInfo **const overallProgressInfo)
 {
     *status = SyncResult::Undefined;
     *unresolvedConflicts = false;
 
-    int cnt = folders.count();
+    const auto cnt = folders.count();
 
-    // if one folder: show the state of the one folder.
+    // if one folder: show the state of the one folder along with the sync status.
     // if more folders:
     // if one of them has an error -> show error
     // if one is paused, but others ok, show ok
     // do not show "problem" in the tray
+    // and do not show sync status
     //
     if (cnt == 1) {
-        Folder *folder = folders.at(0);
+        const auto folder = folders.at(0);
         if (folder) {
             auto syncResult = folder->syncResult();
             if (folder->syncPaused()) {
@@ -1541,62 +1647,74 @@ void FolderMan::trayOverallStatus(const QList<Folder *> &folders,
                 }
             }
             *unresolvedConflicts = syncResult.hasUnresolvedConflicts();
+            *overallProgressInfo = folder->syncEngine().progressInfo();
         }
     } else {
-        int errorsSeen = 0;
-        int goodSeen = 0;
-        int abortOrPausedSeen = 0;
-        int runSeen = 0;
-        int various = 0;
+        auto errorsSeen = false;
+        auto goodSeen = false;
+        auto abortOrPausedSeen = false;
+        auto runSeen = false;
+        auto various = false;
 
         for (const Folder *folder : qAsConst(folders)) {
-            SyncResult folderResult = folder->syncResult();
+            // We've already seen an error, worst case met.
+            // No need to check the remaining folders.
+            if (errorsSeen) {
+                break;
+            }
+
+            const auto folderResult = folder->syncResult();
+
             if (folder->syncPaused()) {
-                abortOrPausedSeen++;
+                abortOrPausedSeen = true;
             } else {
-                SyncResult::Status syncStatus = folderResult.status();
+                const auto syncStatus = folderResult.status();
 
                 switch (syncStatus) {
                 case SyncResult::Undefined:
                 case SyncResult::NotYetStarted:
-                    various++;
+                    various = true;
                     break;
                 case SyncResult::SyncPrepare:
                 case SyncResult::SyncRunning:
-                    runSeen++;
+                    runSeen = true;
                     break;
                 case SyncResult::Problem: // don't show the problem icon in tray.
                 case SyncResult::Success:
-                    goodSeen++;
+                    goodSeen = true;
                     break;
                 case SyncResult::Error:
                 case SyncResult::SetupError:
-                    errorsSeen++;
+                    errorsSeen = true;
                     break;
                 case SyncResult::SyncAbortRequested:
                 case SyncResult::Paused:
-                    abortOrPausedSeen++;
+                    abortOrPausedSeen = true;
                     // no default case on purpose, check compiler warnings
                 }
             }
-            if (folderResult.hasUnresolvedConflicts())
+
+            if (folderResult.hasUnresolvedConflicts()) {
                 *unresolvedConflicts = true;
+            }
         }
-        if (errorsSeen > 0) {
+
+        if (errorsSeen) {
             *status = SyncResult::Error;
-        } else if (abortOrPausedSeen > 0 && abortOrPausedSeen == cnt) {
+        } else if (abortOrPausedSeen) {
             // only if all folders are paused
             *status = SyncResult::Paused;
-        } else if (runSeen > 0) {
+        } else if (runSeen) {
             *status = SyncResult::SyncRunning;
-        } else if (goodSeen > 0) {
+        } else if (goodSeen) {
             *status = SyncResult::Success;
+        } else if (various) {
+            *status = SyncResult::Undefined;
         }
     }
 }
 
-QString FolderMan::trayTooltipStatusString(
-    SyncResult::Status syncStatus, bool hasUnresolvedConflicts, bool paused)
+QString FolderMan::trayTooltipStatusString(SyncResult::Status syncStatus, bool hasUnresolvedConflicts, bool paused, ProgressInfo *const progress)
 {
     QString folderMessage;
     switch (syncStatus) {
@@ -1610,6 +1728,35 @@ QString FolderMan::trayTooltipStatusString(
         folderMessage = tr("Preparing for sync.");
         break;
     case SyncResult::SyncRunning:
+        if (progress && progress->status() == ProgressInfo::Propagation) {
+            const auto estimatedEta = progress->totalProgress().estimatedEta;
+            if (progress->totalSize() == 0) {
+                qint64 currentFile = progress->currentFile();
+                qint64 totalFileCount = qMax(progress->totalFiles(), currentFile);
+                if (progress->trustEta()) {
+                    if (estimatedEta == 0) {
+                        folderMessage = tr("Syncing %1 of %2 (A few seconds left)").arg(currentFile).arg(totalFileCount);
+                    } else {
+                        folderMessage =
+                            tr("Syncing %1 of %2 (%3 left)").arg(currentFile).arg(totalFileCount).arg(Utility::durationToDescriptiveString1(estimatedEta));
+                    }
+                } else {
+                    folderMessage = tr("Syncing %1 of %2").arg(currentFile).arg(totalFileCount);
+                }
+            } else {
+                QString totalSizeStr = Utility::octetsToString(progress->totalSize());
+                if (progress->trustEta()) {
+                    if (estimatedEta == 0) {
+                        folderMessage = tr("Syncing %1 (A few seconds left)").arg(totalSizeStr);
+                    } else {
+                        folderMessage = tr("Syncing %1 (%2 left)").arg(totalSizeStr, Utility::durationToDescriptiveString1(estimatedEta));
+                    }
+                } else {
+                    folderMessage = tr("Syncing %1").arg(totalSizeStr);
+                }
+            }
+            break;
+        }
         folderMessage = tr("Sync is running.");
         break;
     case SyncResult::Success:
@@ -1651,21 +1798,31 @@ static QString checkPathValidityRecursive(const QString &path)
 #endif
     const QFileInfo selFile(path);
 
-    if (!selFile.exists()) {
+    if (!FileSystem::fileExists(path)) {
         QString parentPath = selFile.dir().path();
         if (parentPath != path)
             return checkPathValidityRecursive(parentPath);
         return FolderMan::tr("The selected path does not exist!");
     }
 
-    if (!selFile.isDir()) {
+    if (!FileSystem::isDir(path)) {
         return FolderMan::tr("The selected path is not a folder!");
     }
 
-    if (!selFile.isWritable()) {
+    #ifdef Q_OS_WIN
+    if (!FileSystem::isWritable(path)) {
+        // isWritable() doesn't cover all NTFS permissions
+        // try creating and removing a test file, and make sure it is excluded from sync
+        if (!Utility::canCreateFileInPath(path)) {
+            return FolderMan::tr("You have no permission to write to the selected folder!");
+        }
+    }
+    #else
+    if (!FileSystem::isWritable(path)) {
         return FolderMan::tr("You have no permission to write to the selected folder!");
     }
-    return QString();
+    #endif
+    return {};
 }
 
 // QFileInfo::canonicalPath returns an empty string if the file does not exist.
@@ -1674,7 +1831,7 @@ static QString checkPathValidityRecursive(const QString &path)
 static QString canonicalPath(const QString &path)
 {
     QFileInfo selFile(path);
-    if (!selFile.exists()) {
+    if (!FileSystem::fileExists(path)) {
         const auto parentPath = selFile.dir().path();
 
         // It's possible for the parentPath to match the path
@@ -1729,7 +1886,7 @@ QPair<FolderMan::PathValidityResult, QString> FolderMan::checkPathValidityForNew
             return result;
         }
 
-        // if both pathes are equal, the server url needs to be different
+        // if both paths are equal, the server url needs to be different
         // otherwise it would mean that a new connection from the same local folder
         // to the same account is added which is not wanted. The account must differ.
         if (serverUrl.isValid() && !differentPaths) {
@@ -1767,7 +1924,7 @@ QString FolderMan::findGoodPathForNewSyncFolder(const QString &basePath, const Q
     int attempt = 1;
     forever {
         const auto isGood = FolderMan::instance()->checkPathValidityForNewFolder(folder, serverUrl).second.isEmpty() &&
-            (allowExisting == GoodPathStrategy::AllowOverrideExistingPath || !QFileInfo::exists(folder));
+            (allowExisting == GoodPathStrategy::AllowOverrideExistingPath || !FileSystem::fileExists(folder));
         if (isGood) {
             break;
         }
