@@ -46,12 +46,6 @@
 
 #include <KZip>
 
-#define QTLEGACY (QT_VERSION < QT_VERSION_CHECK(5,9,0))
-
-#if !(QTLEGACY)
-#include <QOperatingSystemVersion>
-#endif
-
 namespace {
 struct ZipEntry {
     QString localFilename;
@@ -73,11 +67,24 @@ ZipEntry fileInfoToLogZipEntry(const QFileInfo &info)
     return entry;
 }
 
-ZipEntry syncFolderToZipEntry(OCC::Folder *f)
+QVector<ZipEntry> syncFolderToDatabaseZipEntry(OCC::Folder *f)
 {
+    QVector<ZipEntry> result;
+
     const auto journalPath = f->journalDb()->databaseFilePath();
     const auto journalInfo = QFileInfo(journalPath);
-    return fileInfoToZipEntry(journalInfo);
+    const auto walJournalInfo = QFileInfo(journalPath + "-wal");
+    const auto shmJournalInfo = QFileInfo(journalPath + "-shm");
+
+    result += fileInfoToZipEntry(journalInfo);
+    if (walJournalInfo.exists()) {
+        result += fileInfoToZipEntry(walJournalInfo);
+    }
+    if (shmJournalInfo.exists()) {
+        result += fileInfoToZipEntry(shmJournalInfo);
+    }
+
+    return result;
 }
 
 QVector<ZipEntry> createDebugArchiveFileList()
@@ -100,15 +107,28 @@ QVector<ZipEntry> createDebugArchiveFileList()
     }
 
     const auto folders = OCC::FolderMan::instance()->map().values();
-    std::transform(std::cbegin(folders), std::cend(folders),
-                   std::back_inserter(list),
-                   syncFolderToZipEntry);
+    std::for_each(std::cbegin(folders), std::cend(folders),
+                  [&list] (auto &folderIt) {
+                      const auto &newEntries = syncFolderToDatabaseZipEntry(folderIt);
+                      std::copy(std::cbegin(newEntries), std::cend(newEntries), std::back_inserter(list));
+                  });
 
     return list;
 }
 
-void createDebugArchive(const QString &filename)
+bool createDebugArchive(const QString &filename)
 {
+    const auto fileInfo = QFileInfo(filename);
+    const auto dirInfo = QFileInfo(fileInfo.dir().absolutePath());
+    if (!dirInfo.isWritable()) {
+        QMessageBox::critical(
+            nullptr,
+            QObject::tr("Failed to create debug archive"),
+            QObject::tr("Could not create debug archive in selected location!")
+        );
+        return false;
+    }
+
     const auto entries = createDebugArchiveFileList();
 
     KZip zip(filename);
@@ -127,7 +147,9 @@ void createDebugArchive(const QString &filename)
     zip.prepareWriting("__nextcloud_client_buildinfo.txt", {}, {}, buildInfo.size());
     zip.writeData(buildInfo, buildInfo.size());
     zip.finishWriting(buildInfo.size());
+    return true;
 }
+
 }
 
 namespace OCC {
@@ -177,8 +199,6 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     });
 
     loadMiscSettings();
-    // updater info now set in: customizeStyle
-    //slotUpdateInfo();
 
     // misc
     connect(_ui->monoIconsCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
@@ -197,11 +217,7 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     // Hide on non-Windows, or WindowsVersion < 10.
     // The condition should match the default value of ConfigFile::showInExplorerNavigationPane.
 #ifdef Q_OS_WIN
-    #if QTLEGACY
-        if (QSysInfo::windowsVersion() < QSysInfo::WV_WINDOWS10)
-    #else
         if (QOperatingSystemVersion::current() < QOperatingSystemVersion::Windows10)
-    #endif
             _ui->showInExplorerNavigationPaneCheckBox->setVisible(false);
 #else
     // Hide on non-Windows
@@ -268,13 +284,23 @@ void GeneralSettings::loadMiscSettings()
     _ui->stopExistingFolderNowBigSyncCheckBox->setChecked(_ui->existingFolderLimitCheckBox->isChecked() && cfgFile.stopSyncingExistingFoldersOverLimit());
     _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
+
+#if defined(BUILD_UPDATER)
+    const auto validUpdateChannels = cfgFile.validUpdateChannels();
+    _ui->updateChannel->clear();
+    _ui->updateChannel->addItems(validUpdateChannels);
+    const auto currentUpdateChannelIndex = validUpdateChannels.indexOf(cfgFile.currentUpdateChannel());
+    _ui->updateChannel->setCurrentIndex(currentUpdateChannelIndex != -1? currentUpdateChannelIndex : 0);
+    connect(_ui->updateChannel, &QComboBox::currentTextChanged, this, &GeneralSettings::slotUpdateChannelChanged);
+#endif
 }
 
 #if defined(BUILD_UPDATER)
 void GeneralSettings::slotUpdateInfo()
 {
+    ConfigFile config;
     const auto updater = Updater::instance();
-    if (ConfigFile().skipUpdateCheck() || !updater) {
+    if (config.skipUpdateCheck() || !updater) {
         // updater disabled on compile
         _ui->updatesContainer->setVisible(false);
         return;
@@ -284,22 +310,20 @@ void GeneralSettings::slotUpdateInfo()
         connect(_ui->updateButton,
                 &QAbstractButton::clicked,
                 this,
-
                 &GeneralSettings::slotUpdateCheckNow,
                 Qt::UniqueConnection);
         connect(_ui->autoCheckForUpdatesCheckBox, &QAbstractButton::toggled, this,
                 &GeneralSettings::slotToggleAutoUpdateCheck, Qt::UniqueConnection);
-        _ui->autoCheckForUpdatesCheckBox->setChecked(ConfigFile().autoUpdateCheck());
+        _ui->autoCheckForUpdatesCheckBox->setChecked(config.autoUpdateCheck());
     }
 
     // Note: the sparkle-updater is not an OCUpdater
-    auto *ocupdater = qobject_cast<OCUpdater *>(updater);
+    const auto ocupdater = qobject_cast<OCUpdater *>(updater);
     if (ocupdater) {
         connect(ocupdater, &OCUpdater::downloadStateChanged, this, &GeneralSettings::slotUpdateInfo, Qt::UniqueConnection);
         connect(_ui->restartButton, &QAbstractButton::clicked, ocupdater, &OCUpdater::slotStartInstaller, Qt::UniqueConnection);
-        connect(_ui->restartButton, &QAbstractButton::clicked, qApp, &QApplication::quit, Qt::UniqueConnection);
 
-        QString status = ocupdater->statusString(OCUpdater::UpdateStatusStringFormat::Html);
+        auto status = ocupdater->statusString(OCUpdater::UpdateStatusStringFormat::Html);
         Theme::replaceLinkColorStringBackgroundAware(status);
 
         _ui->updateStateLabel->setOpenExternalLinks(false);
@@ -307,9 +331,7 @@ void GeneralSettings::slotUpdateInfo()
             Utility::openBrowser(QUrl(link));
         });
         _ui->updateStateLabel->setText(status);
-
         _ui->restartButton->setVisible(ocupdater->downloadState() == OCUpdater::DownloadComplete);
-
         _ui->updateButton->setEnabled(ocupdater->downloadState() != OCUpdater::CheckingServer &&
                                       ocupdater->downloadState() != OCUpdater::Downloading &&
                                       ocupdater->downloadState() != OCUpdater::DownloadComplete);
@@ -326,57 +348,67 @@ void GeneralSettings::slotUpdateInfo()
         _ui->updateButton->setEnabled(enableUpdateButton);
     }
 #endif
-
-    // Channel selection
-    _ui->updateChannel->setCurrentIndex(ConfigFile().updateChannel() == "beta" ? 1 : 0);
-    connect(_ui->updateChannel, &QComboBox::currentTextChanged,
-        this, &GeneralSettings::slotUpdateChannelChanged, Qt::UniqueConnection);
 }
 
 void GeneralSettings::slotUpdateChannelChanged()
 {
     const auto updateChannelToLocalized = [](const QString &channel) {
-        auto decodedTranslatedChannel = QString{};
-
         if (channel == QStringLiteral("stable")) {
-            decodedTranslatedChannel = tr("stable");
-        } else if (channel == QStringLiteral("beta")) {
-            decodedTranslatedChannel = tr("beta");
+            return tr("stable");
         }
 
-        return decodedTranslatedChannel;
+        if (channel == QStringLiteral("beta")) {
+            return tr("beta");
+        }
+
+        if (channel == QStringLiteral("daily")) {
+            return tr("daily");
+        }
+
+        if (channel == QStringLiteral("enterprise")) {
+            return tr("enterprise");
+        }
+
+        return QString{};
     };
 
     const auto updateChannelFromLocalized = [](const int index) {
-        if (index == 1) {
+        switch(index) {
+        case 1:
             return QStringLiteral("beta");
+            break;
+        case 2:
+            return QStringLiteral("daily");
+            break;
+        case 3:
+            return QStringLiteral("enterprise");
+            break;
+        default:
+            return QStringLiteral("stable");
         }
-
-        return QStringLiteral("stable");
     };
 
+    ConfigFile configFile;
     const auto channel = updateChannelFromLocalized(_ui->updateChannel->currentIndex());
-    if (channel == ConfigFile().updateChannel()) {
+    if (channel == configFile.currentUpdateChannel()) {
         return;
     }
 
+    const auto enterprise = configFile.validUpdateChannels().contains("enterprise") ? tr("- enterprise: contains stable versions for customers.\n",
+                                                                                         "description of enterprise update channel for enterprise customers")
+                                                                                    : "";
     auto msgBox = new QMessageBox(
         QMessageBox::Warning,
-        tr("Change update channel?"),
-        tr("The update channel determines which client updates will be offered "
-           "for installation. The \"stable\" channel contains only upgrades that "
-           "are considered reliable, while the versions in the \"beta\" channel "
-           "may contain newer features and bugfixes, but have not yet been tested "
-           "thoroughly."
-           "\n\n"
-           "Note that this selects only what pool upgrades are taken from, and that "
-           "there are no downgrades: So going back from the beta channel to "
-           "the stable channel usually cannot be done immediately and means waiting "
-           "for a stable version that is newer than the currently installed beta "
-           "version."),
+        tr("Changing update channel?"),
+        tr("The channel determines which upgrades will be offered to install:\n"
+           "- stable: contains tested versions considered reliable\n"
+           "- beta: contains versions with new features that may not be tested thoroughly\n"
+           "- daily: contains versions created daily only for testing and development\n"
+           "%1\n"
+           "Downgrading versions is not possible immediately: changing from beta to stable means waiting for the new stable version.").arg(enterprise),
         QMessageBox::NoButton,
         this);
-    auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
+    const auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
     msgBox->addButton(tr("Cancel"), QMessageBox::RejectRole);
     connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton, updateChannelToLocalized] {
         msgBox->deleteLater();
@@ -393,7 +425,7 @@ void GeneralSettings::slotUpdateChannelChanged()
             }
 #endif
         } else {
-            _ui->updateChannel->setCurrentText(updateChannelToLocalized(ConfigFile().updateChannel()));
+            _ui->updateChannel->setCurrentText(updateChannelToLocalized(ConfigFile().currentUpdateChannel()));
         }
     });
     msgBox->open();
@@ -498,13 +530,24 @@ void GeneralSettings::slotIgnoreFilesEditor()
 
 void GeneralSettings::slotCreateDebugArchive()
 {
-    const auto filename = QFileDialog::getSaveFileName(this, tr("Create Debug Archive"), QString(), tr("Zip Archives") + " (*.zip)");
+    const auto filename = QFileDialog::getSaveFileName(
+        this,
+        tr("Create Debug Archive"),
+        QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation),
+        tr("Zip Archives") + " (*.zip)"
+    );
+
     if (filename.isEmpty()) {
         return;
     }
 
-    createDebugArchive(filename);
-    QMessageBox::information(this, tr("Debug Archive Created"), tr("Debug archive is created at %1").arg(filename));
+    if (createDebugArchive(filename)) {
+        QMessageBox::information(
+            this,
+            tr("Debug Archive Created"),
+            tr("Debug archive is created at %1").arg(filename)
+        );
+    }
 }
 
 void GeneralSettings::slotShowLegalNotice()
