@@ -86,7 +86,6 @@ void UpdaterScheduler::slotTimerFired()
 OCUpdater::OCUpdater(const QUrl &url)
     : Updater()
     , _updateUrl(url)
-    , _state(Unknown)
     , _accessManager(new AccessManager(this))
     , _timeoutWatchdog(new QTimer(this))
 {
@@ -150,7 +149,7 @@ QString OCUpdater::statusString(UpdateStatusStringFormat format) const
 
     switch (downloadState()) {
     case Downloading:
-        return tr("Downloading %1. Please wait …").arg(updateVersion);
+        return tr("Downloading %1 …").arg(updateVersion);
     case DownloadComplete:
         return tr("%1 available. Restart application to start the update.").arg(updateVersion);
     case DownloadFailed: {
