@@ -16,6 +16,8 @@
 #include "config.h"
 #include "common/utility.h"
 #include "version.h"
+#include "configfile.h"
+#include "common/vfs.h"
 
 #include <QtCore>
 #ifndef TOKEN_AUTH_ONLY
@@ -23,8 +25,13 @@
 #include <QStyle>
 #include <QApplication>
 #endif
+#include <QGuiApplication>
+#include <QStyleHints>
 #include <QSslSocket>
 #include <QSvgRenderer>
+#include <QPainter>
+#include <QJsonDocument>
+#include <QJsonArray>
 
 #include "nextcloudtheme.h"
 
@@ -79,31 +86,31 @@ QString Theme::statusHeaderText(SyncResult::Status status) const
 
     switch (status) {
     case SyncResult::Undefined:
-        resultStr = QCoreApplication::translate("theme", "Status undefined");
+        resultStr = QCoreApplication::translate("theme", "Sync status is unknown");
         break;
     case SyncResult::NotYetStarted:
-        resultStr = QCoreApplication::translate("theme", "Waiting to start sync");
+        resultStr = QCoreApplication::translate("theme", "Waiting to start syncing");
         break;
     case SyncResult::SyncRunning:
         resultStr = QCoreApplication::translate("theme", "Sync is running");
         break;
     case SyncResult::Success:
-        resultStr = QCoreApplication::translate("theme", "Sync Success");
+        resultStr = QCoreApplication::translate("theme", "Sync was successful");
         break;
     case SyncResult::Problem:
-        resultStr = QCoreApplication::translate("theme", "Sync Success, some files were ignored.");
+        resultStr = QCoreApplication::translate("theme", "Sync was successful but some files were ignored");
         break;
     case SyncResult::Error:
-        resultStr = QCoreApplication::translate("theme", "Sync Error");
+        resultStr = QCoreApplication::translate("theme", "Error occurred during sync");
         break;
     case SyncResult::SetupError:
-        resultStr = QCoreApplication::translate("theme", "Setup Error");
+        resultStr = QCoreApplication::translate("theme", "Error occurred during setup");
         break;
     case SyncResult::SyncPrepare:
         resultStr = QCoreApplication::translate("theme", "Preparing to sync");
         break;
     case SyncResult::SyncAbortRequested:
-        resultStr = QCoreApplication::translate("theme", "Aborting …");
+        resultStr = QCoreApplication::translate("theme", "Stopping sync");
         break;
     case SyncResult::Paused:
         resultStr = QCoreApplication::translate("theme", "Sync is paused");
@@ -114,7 +121,7 @@ QString Theme::statusHeaderText(SyncResult::Status status) const
 
 bool Theme::isBranded() const
 {
-    return appNameGUI() != QStringLiteral("Nextcloud");
+    return (appNameGUI() != QStringLiteral("Nextcloud") && NEXTCLOUD_DEV == 0);
 }
 
 QString Theme::appNameGUI() const
@@ -137,14 +144,69 @@ QUrl Theme::stateOfflineImageSource() const
     return imagePathToUrl(themeImagePath("state-offline", 16));
 }
 
+QUrl Theme::statusOnlineImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-online", 16));
+}
+
+QUrl Theme::statusDoNotDisturbImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-dnd", 16));
+}
+
+QUrl Theme::statusAwayImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-away", 16));
+}
+
+QUrl Theme::statusInvisibleImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-invisible", 64));
+}
+
+QUrl Theme::syncStatusOk() const
+{
+    return imagePathToUrl(themeImagePath("state-ok", 16));
+}
+
+QUrl Theme::syncStatusError() const
+{
+    return imagePathToUrl(themeImagePath("state-error", 16));
+}
+
+QUrl Theme::syncStatusRunning() const
+{
+    return imagePathToUrl(themeImagePath("state-sync", 16));
+}
+
+QUrl Theme::syncStatusPause() const
+{
+    return imagePathToUrl(themeImagePath("state-pause", 16));
+}
+
+QUrl Theme::syncStatusWarning() const
+{
+    return imagePathToUrl(themeImagePath("state-warning", 16));
+}
+
+QUrl Theme::folderOffline() const
+{
+    return imagePathToUrl(themeImagePath("state-offline"));
+}
+
 QString Theme::version() const
 {
     return MIRALL_VERSION_STRING;
 }
 
+QString Theme::versionSuffix() const
+{
+    return QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION_SUFFIX));
+}
+
 QString Theme::configFileName() const
 {
-    return QStringLiteral(APPLICATION_EXECUTABLE ".cfg");
+    return QStringLiteral(APPLICATION_CONFIG_NAME ".cfg");
 }
 
 #ifndef TOKEN_AUTH_ONLY
@@ -175,7 +237,7 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray) const
             return cached = QIcon::fromTheme(name);
         }
 
-        const auto svgName = QString::fromLatin1(":/client/theme/%1/%2.svg").arg(flavor).arg(name);
+        const QString svgName = QString(Theme::themePrefix) + QString::fromLatin1("%1/%2.svg").arg(flavor).arg(name);
         QSvgRenderer renderer(svgName);
         const auto createPixmapFromSvg = [&renderer] (int size) {
             QImage img(size, size, QImage::Format_ARGB32);
@@ -186,7 +248,7 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray) const
         };
 
         const auto loadPixmap = [flavor, name] (int size) {
-            const auto pixmapName = QString::fromLatin1(":/client/theme/%1/%2-%3.png").arg(flavor).arg(name).arg(size);
+            const QString pixmapName = QString(Theme::themePrefix) + QString::fromLatin1("%1/%2-%3.png").arg(flavor).arg(name).arg(size);
             return QPixmap(pixmapName);
         };
 
@@ -210,11 +272,9 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray) const
     }
 
 #ifdef Q_OS_MAC
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
     // This defines the icon as a template and enables automatic macOS color handling
     // See https://bugreports.qt.io/browse/QTBUG-42109
     cached.setIsMask(_mono && sysTray);
-#endif
 #endif
 
     return cached;
@@ -227,8 +287,8 @@ QString Theme::themeImagePath(const QString &name, int size, bool sysTray) const
 
     // branded client may have several sizes of the same icon
     const QString filePath = (useSvg || size <= 0)
-            ? QString::fromLatin1(":/client/theme/%1/%2").arg(flavor).arg(name)
-            : QString::fromLatin1(":/client/theme/%1/%2-%3").arg(flavor).arg(name).arg(size);
+            ? QString(Theme::themePrefix) + QString::fromLatin1("%1/%2").arg(flavor).arg(name)
+            : QString(Theme::themePrefix) + QString::fromLatin1("%1/%2-%3").arg(flavor).arg(name).arg(size);
 
     const QString svgPath = filePath + ".svg";
     if (useSvg) {
@@ -244,18 +304,22 @@ QString Theme::themeImagePath(const QString &name, int size, bool sysTray) const
     }
 }
 
+bool Theme::isHidpi(QPaintDevice *dev)
+{
+    const auto devicePixelRatio = dev ? dev->devicePixelRatio() : qApp->primaryScreen()->devicePixelRatio();
+    return devicePixelRatio > 1;
+}
+
 QIcon Theme::uiThemeIcon(const QString &iconName, bool uiHasDarkBg) const
 {
-    QString themeResBasePath = ":/client/theme/";
-    QString iconPath = themeResBasePath + (uiHasDarkBg?"white/":"black/") + iconName;
+    QString iconPath = QString(Theme::themePrefix) + (uiHasDarkBg ? "white/" : "black/") + iconName;
     std::string icnPath = iconPath.toUtf8().constData();
     return QIcon(QPixmap(iconPath));
 }
 
 QString Theme::hidpiFileName(const QString &fileName, QPaintDevice *dev)
 {
-    qreal devicePixelRatio = dev ? dev->devicePixelRatio() : qApp->primaryScreen()->devicePixelRatio();
-    if (devicePixelRatio <= 1.0) {
+    if (!Theme::isHidpi(dev)) {
         return fileName;
     }
     // try to find a 2x version
@@ -272,12 +336,57 @@ QString Theme::hidpiFileName(const QString &fileName, QPaintDevice *dev)
     return fileName;
 }
 
+QString Theme::hidpiFileName(const QString &iconName, const QColor &backgroundColor, QPaintDevice *dev)
+{
+    const auto isDarkBackground = Theme::isDarkColor(backgroundColor);
+
+    const QString iconPath = QString(Theme::themePrefix) + (isDarkBackground ? "white/" : "black/") + iconName;
+
+    return Theme::hidpiFileName(iconPath, dev);
+}
+
 
 #endif
 
 Theme::Theme()
     : QObject(nullptr)
 {
+#if defined(Q_OS_WIN)
+    // Windows does not provide a dark theme for Win32 apps so let's come up with a palette
+    // Credit to https://github.com/Jorgen-VikingGod/Qt-Frameless-Window-DarkStyle
+
+    reserveDarkPalette.setColor(QPalette::Window, QColor(53, 53, 53));
+    reserveDarkPalette.setColor(QPalette::WindowText, Qt::white);
+    reserveDarkPalette.setColor(QPalette::Disabled, QPalette::WindowText,
+                                QColor(127, 127, 127));
+    reserveDarkPalette.setColor(QPalette::Base, QColor(42, 42, 42));
+    reserveDarkPalette.setColor(QPalette::AlternateBase, QColor(66, 66, 66));
+    reserveDarkPalette.setColor(QPalette::ToolTipBase, Qt::white);
+    reserveDarkPalette.setColor(QPalette::ToolTipText, QColor(53, 53, 53));
+    reserveDarkPalette.setColor(QPalette::Text, Qt::white);
+    reserveDarkPalette.setColor(QPalette::Disabled, QPalette::Text, QColor(127, 127, 127));
+    reserveDarkPalette.setColor(QPalette::Dark, QColor(35, 35, 35));
+    reserveDarkPalette.setColor(QPalette::Shadow, QColor(20, 20, 20));
+    reserveDarkPalette.setColor(QPalette::Button, QColor(53, 53, 53));
+    reserveDarkPalette.setColor(QPalette::ButtonText, Qt::white);
+    reserveDarkPalette.setColor(QPalette::Disabled, QPalette::ButtonText,
+                                QColor(127, 127, 127));
+    reserveDarkPalette.setColor(QPalette::BrightText, Qt::red);
+    reserveDarkPalette.setColor(QPalette::Link, QColor(42, 130, 218));
+    reserveDarkPalette.setColor(QPalette::Highlight, QColor(42, 130, 218));
+    reserveDarkPalette.setColor(QPalette::Disabled, QPalette::Highlight, QColor(80, 80, 80));
+    reserveDarkPalette.setColor(QPalette::HighlightedText, Qt::white);
+    reserveDarkPalette.setColor(QPalette::Disabled, QPalette::HighlightedText,
+                                QColor(127, 127, 127));
+#endif
+
+#ifdef APPLICATION_SERVER_URL_ENFORCE
+    _forceOverrideServerUrl = true;
+#endif
+#ifdef APPLICATION_SERVER_URL
+    _overrideServerUrl = QString::fromLatin1(APPLICATION_SERVER_URL);
+    updateMultipleOverrideServers();
+#endif
 }
 
 // If this option returns true, the client only supports one folder to sync.
@@ -308,26 +417,67 @@ QString Theme::helpUrl() const
 
 QString Theme::conflictHelpUrl() const
 {
-    auto baseUrl = helpUrl();
-    if (baseUrl.isEmpty())
+    const auto baseUrl = helpUrl();
+    if (baseUrl.isEmpty()) {
         return QString();
-    if (!baseUrl.endsWith('/'))
-        baseUrl.append('/');
-    return baseUrl + QStringLiteral("conflicts.html");
+    }
+
+    return Utility::trailingSlashPath(baseUrl) + QStringLiteral("conflicts.html");
 }
 
 QString Theme::overrideServerUrl() const
 {
-#ifdef APPLICATION_SERVER_URL
-    return QString::fromLatin1(APPLICATION_SERVER_URL);
-#else
-    return QString();
-#endif
+    return _overrideServerUrl;
 }
 
 bool Theme::forceOverrideServerUrl() const
 {
-#ifdef APPLICATION_SERVER_URL_ENFORCE
+    return _forceOverrideServerUrl;
+}
+
+void Theme::updateMultipleOverrideServers()
+{
+    const auto json = overrideServerUrl().toUtf8();
+    const auto doc = QJsonDocument::fromJson(json);
+    _multipleOverrideServers = doc.isArray() && !doc.array().empty();
+}
+
+bool Theme::multipleOverrideServers() const
+{
+    return _multipleOverrideServers;
+}
+
+bool Theme::isVfsEnabled() const
+{
+    return _isVfsEnabled;
+}
+
+bool Theme::startLoginFlowAutomatically() const
+{
+    return _startLoginFlowAutomatically;
+}
+
+bool Theme::enableStaplingOCSP() const
+{
+#ifdef APPLICATION_OCSP_STAPLING_ENABLED
+    return true;
+#else
+    return false;
+#endif
+}
+
+bool Theme::forbidBadSSL() const
+{
+#ifdef APPLICATION_FORBID_BAD_SSL
+    return true;
+#else
+    return false;
+#endif
+}
+
+bool Theme::doNotUseProxy() const
+{
+#ifdef DO_NOT_USE_PROXY
     return true;
 #else
     return false;
@@ -369,7 +519,7 @@ bool Theme::systrayUseMonoIcons() const
 
 bool Theme::monoIconsAvailable() const
 {
-    QString themeDir = QString::fromLatin1(":/client/theme/%1/").arg(Theme::instance()->systrayIconFlavor(true));
+    QString themeDir = QString(Theme::themePrefix) + QString::fromLatin1("%1/").arg(Theme::instance()->systrayIconFlavor(true));
     return QDir(themeDir).exists();
 }
 
@@ -401,7 +551,7 @@ QString Theme::gitSHA1() const
     const QString githubPrefix(QLatin1String(
         "https://github.com/nextcloud/desktop/commit/"));
     const QString gitSha1(QLatin1String(GIT_SHA1));
-    devString = QCoreApplication::translate("nextcloudTheme::about()",
+    devString = QCoreApplication::translate("nextcloudTheme::aboutInfo()",
         "<p><small>Built from Git revision <a href=\"%1\">%2</a>"
         " on %3, %4 using Qt %5, %6</small></p>")
                     .arg(githubPrefix + gitSha1)
@@ -414,20 +564,31 @@ QString Theme::gitSHA1() const
     return devString;
 }
 
+QString Theme::aboutInfo() const
+{
+    // Shorten Qt's OS name: "macOS Mojave (10.14)" -> "macOS"
+    QStringList osStringList = Utility::platformName().split(QLatin1Char(' '));
+    QString osName = osStringList.at(0);
+
+    //: Example text: "<p>Nextcloud Desktop Client</p>"   (%1 is the application name)
+    auto devString = QString(tr("<p>%1 Desktop Client</p><p>Version %1. For more information please click <a href='%2'>here</a>.</p>") + QStringLiteral(" (%3)"))
+            .arg(APPLICATION_NAME, QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION)), helpUrl(), osName);
+                      
+
+    devString += tr("<p><small>Using virtual files plugin: %1</small></p>").arg(Vfs::modeToString(bestAvailableVfsMode()));
+    devString += QStringLiteral("<br>%1").arg(QSysInfo::productType() % QLatin1Char('-') % QSysInfo::kernelVersion());
+
+    return devString;
+}
+
 QString Theme::about() const
 {
     // Shorten Qt's OS name: "macOS Mojave (10.14)" -> "macOS"
     QStringList osStringList = Utility::platformName().split(QLatin1Char(' '));
     QString osName = osStringList.at(0);
 
-    QString devString;
     //: Example text: "<p>Nextcloud Desktop Client</p>"   (%1 is the application name)
-    devString = tr("<p>%1 Desktop Client</p>")
-              .arg(APPLICATION_NAME);
-
-    devString += tr("<p>Version %1. For more information please click <a href='%2'>here</a>.</p>")
-              .arg(QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION)) + QString(" (%1)").arg(osName))
-              .arg(helpUrl());
+    const auto devString = tr("<p>%1 desktop client %2</p>").arg(APPLICATION_NAME, QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION)));
 
     return devString;
 }
@@ -439,7 +600,7 @@ QString Theme::aboutDetails() const
               .arg(MIRALL_VERSION_STRING)
               .arg(helpUrl());
 
-    devString += tr("<p>This release was supplied by %1</p>")
+    devString += tr("<p>This release was supplied by %1.</p>")
               .arg(APPLICATION_VENDOR);
 
     devString += gitSHA1();
@@ -468,7 +629,7 @@ QVariant Theme::customMedia(CustomMediaType type)
         break;
     }
 
-    QString imgPath = QString::fromLatin1(":/client/theme/colored/%1.png").arg(key);
+    QString imgPath = QString(Theme::themePrefix) + QString::fromLatin1("colored/%1.png").arg(key);
     if (QFile::exists(imgPath)) {
         QPixmap pix(imgPath);
         if (pix.isNull()) {
@@ -536,11 +697,34 @@ QColor Theme::wizardHeaderBackgroundColor() const
     return {APPLICATION_WIZARD_HEADER_BACKGROUND_COLOR};
 }
 
+QPixmap Theme::wizardApplicationLogo() const
+{
+    if (!Theme::isBranded()) {
+        return QPixmap(Theme::hidpiFileName(QString(Theme::themePrefix) + "colored/wizard-nextcloud.png"));
+    }
+#ifdef APPLICATION_WIZARD_USE_CUSTOM_LOGO
+    const auto useSvg = shouldPreferSvg();
+    const QString logoBasePath = QString(Theme::themePrefix) + QStringLiteral("colored/wizard_logo");
+    if (useSvg) {
+        const auto maxHeight = Theme::isHidpi() ? 200 : 100;
+        const auto maxWidth = 2 * maxHeight;
+        const auto icon = QIcon(logoBasePath + ".svg");
+        const auto size = icon.actualSize(QSize(maxWidth, maxHeight));
+        return icon.pixmap(size);
+    } else {
+        return QPixmap(hidpiFileName(logoBasePath + ".png"));
+    }
+#else
+    const auto size = Theme::isHidpi() ?: 200 : 100;
+    return applicationIcon().pixmap(size);
+#endif
+}
+
 QPixmap Theme::wizardHeaderLogo() const
 {
 #ifdef APPLICATION_WIZARD_USE_CUSTOM_LOGO
     const auto useSvg = shouldPreferSvg();
-    const auto logoBasePath = QStringLiteral(":/client/theme/colored/wizard_logo");
+    const QString logoBasePath = QString(Theme::themePrefix) + QStringLiteral("colored/wizard_logo");
     if (useSvg) {
         const auto maxHeight = 64;
         const auto maxWidth = 2 * maxHeight;
@@ -580,16 +764,6 @@ bool Theme::wizardSelectiveSyncDefaultNothing() const
     return false;
 }
 
-QString Theme::webDavPath() const
-{
-    return QLatin1String("remote.php/webdav/");
-}
-
-QString Theme::webDavPathNonShib() const
-{
-    return QLatin1String("remote.php/nonshib-webdav/");
-}
-
 bool Theme::linkSharing() const
 {
     return true;
@@ -636,41 +810,36 @@ QString Theme::quotaBaseFolder() const
     return QLatin1String("/");
 }
 
-QString Theme::oauthClientId() const
-{
-    return "xdXOt13JKxym1B1QcEncf2XDkLAexMBFwiT9j6EfhhHFJhs2KM9jbjTmf8JBXE69";
-}
-
-QString Theme::oauthClientSecret() const
-{
-    return "UBntmLjC2yYCeHwsyj73Uwo9TAaecAetRwMw0xYcvNL9yRdLSUi0hUAHfvCHFeFh";
-}
-
 QString Theme::versionSwitchOutput() const
 {
     QString helpText;
     QTextStream stream(&helpText);
     stream << appName()
            << QLatin1String(" version ")
-           << version() << endl;
+           << version() << Qt::endl;
 #ifdef GIT_SHA1
-    stream << "Git revision " << GIT_SHA1 << endl;
+    stream << "Git revision " << GIT_SHA1 << Qt::endl;
 #endif
-    stream << "Using Qt " << qVersion() << ", built against Qt " << QT_VERSION_STR << endl;
+    stream << "Using Qt " << qVersion() << ", built against Qt " << QT_VERSION_STR << Qt::endl;
 
     if(!QGuiApplication::platformName().isEmpty())
-        stream << "Using Qt platform plugin '" << QGuiApplication::platformName() << "'" << endl;
+        stream << "Using Qt platform plugin '" << QGuiApplication::platformName() << "'" << Qt::endl;
 
-    stream << "Using '" << QSslSocket::sslLibraryVersionString() << "'" << endl;
-    stream << "Running on " << Utility::platformName() << ", " << QSysInfo::currentCpuArchitecture() << endl;
+    stream << "Using '" << QSslSocket::sslLibraryVersionString() << "'" << Qt::endl;
+    stream << "Running on " << Utility::platformName() << ", " << QSysInfo::currentCpuArchitecture() << Qt::endl;
     return helpText;
 }
 
+double Theme::getColorDarkness(const QColor &color)
+{
+    // account for different sensitivity of the human eye to certain colors
+    const double threshold = 1.0 - (0.299 * color.red() + 0.587 * color.green() + 0.114 * color.blue()) / 255.0;
+    return threshold;
+}
+
 bool Theme::isDarkColor(const QColor &color)
 {
-    // account for different sensitivity of the human eye to certain colors
-    double treshold = 1.0 - (0.299 * color.red() + 0.587 * color.green() + 0.114 * color.blue()) / 255.0;
-    return treshold > 0.5;
+    return getColorDarkness(color) > 0.5;
 }
 
 QColor Theme::getBackgroundAwareLinkColor(const QColor &backgroundColor)
@@ -695,7 +864,8 @@ void Theme::replaceLinkColorStringBackgroundAware(QString &linkString)
 
 void Theme::replaceLinkColorString(QString &linkString, const QColor &newColor)
 {
-    linkString.replace(QRegularExpression("(<a href|<a style='color:#([a-zA-Z0-9]{6});' href)"), QString::fromLatin1("<a style='color:%1;' href").arg(newColor.name()));
+    static const QRegularExpression linkRegularExpression("(<a href|<a style='color:#([a-zA-Z0-9]{6});' href)");
+    linkString.replace(linkRegularExpression, QString::fromLatin1("<a style='color:%1;' href").arg(newColor.name()));
 }
 
 QIcon Theme::createColorAwareIcon(const QString &name, const QPalette &palette)
@@ -752,4 +922,79 @@ QPixmap Theme::createColorAwarePixmap(const QString &name)
     return createColorAwarePixmap(name, QGuiApplication::palette());
 }
 
+bool Theme::showVirtualFilesOption() const
+{
+    const auto vfsMode = bestAvailableVfsMode();
+    return ConfigFile().showExperimentalOptions() || vfsMode == Vfs::WindowsCfApi;
+}
+
+bool Theme::enforceVirtualFilesSyncFolder() const
+{
+    const auto vfsMode = bestAvailableVfsMode();
+    return ENFORCE_VIRTUAL_FILES_SYNC_FOLDER && vfsMode != OCC::Vfs::Off;
+}
+
+QColor Theme::defaultColor()
+{
+    return QColor{NEXTCLOUD_BACKGROUND_COLOR};
+}
+
+void Theme::connectToPaletteSignal()
+{
+    if (!_paletteSignalsConnected) {
+        if (const auto ptr = qobject_cast<QGuiApplication *>(QGuiApplication::instance())) {
+            connect(ptr->styleHints(), &QStyleHints::colorSchemeChanged, this, &Theme::darkModeChanged);
+            _paletteSignalsConnected = true;
+        }
+    }
+}
+
+bool Theme::darkMode()
+{
+    connectToPaletteSignal();
+    switch (qGuiApp->styleHints()->colorScheme())
+    {
+    case Qt::ColorScheme::Dark:
+        return true;
+    case Qt::ColorScheme::Light:
+        return false;
+    case Qt::ColorScheme::Unknown:
+        return Theme::isDarkColor(QGuiApplication::palette().window().color());
+    }
+
+    return false;
+}
+
+void Theme::setOverrideServerUrl(const QString &overrideServerUrl)
+{
+    if (_overrideServerUrl != overrideServerUrl) {
+        _overrideServerUrl = overrideServerUrl;
+        updateMultipleOverrideServers();
+        emit overrideServerUrlChanged();
+    }
+}
+void Theme::setForceOverrideServerUrl(bool forceOverride)
+{
+    if (_forceOverrideServerUrl != forceOverride) {
+        _forceOverrideServerUrl = forceOverride;
+        emit forceOverrideServerUrlChanged();
+    }
+}
+
+void Theme::setVfsEnabled(bool enabled)
+{
+    if (_isVfsEnabled != enabled) {
+        _isVfsEnabled = enabled;
+        emit vfsEnabledChanged();
+    }
+}
+
+void Theme::setStartLoginFlowAutomatically(bool startLoginFlowAuto)
+{
+    if (_startLoginFlowAutomatically != startLoginFlowAuto) {
+        _startLoginFlowAutomatically = startLoginFlowAuto;
+        emit startLoginFlowAutomaticallyChanged();
+    }
+}
+
 } // end namespace client
